/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.apache.avalon.excalibur.system.handler;

import org.apache.avalon.excalibur.system.Container;
import org.apache.avalon.excalibur.system.RoleManager;
import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.excalibur.logger.LoggerManager;

/**
 * The DefaultComponentHandler to make sure components are initialized
 * and destroyed correctly.
 *
 * @author <a href="mailto:bloritsch@apache.org">Berin Loritsch</a>
 * @version CVS $Revision: 1.2 $ $Date: 2002/01/29 16:18:53 $
 * @since 4.0
 */
class DefaultComponentHandler
    implements ComponentHandler
{
    /** The instance of the ComponentFactory that creates and disposes of the Component */
    private final ComponentFactory     m_factory;

    /** State management boolean stating whether the Handler is initialized or not */
    private boolean                    m_initialized   = false;

    /** State management boolean stating whether the Handler is disposed or not */
    private boolean                    m_disposed      = false;

    /** Logger for factory */
    private final Logger               m_logger;

    /**
     * Create a ComponentHandler that takes care of hiding the details of
     * whether a Component is ThreadSafe, Poolable, or SingleThreaded.
     * It falls back to SingleThreaded if not specified.
     */
    protected DefaultComponentHandler( final Class componentClass,
                                       final Configuration config,
                                       final ComponentManager manager,
                                       final Context context )
        throws Exception
    {
        RoleManager roles = (RoleManager) context.get( Container.ROLE_MANAGER );
        LoggerManager logkit = (LoggerManager) context.get( Container.LOGGER_MANAGER );

        m_factory = new ComponentFactory( componentClass, config, manager, context, roles, logkit );
        m_logger = logkit.getLoggerForCategory("system.handler.factory");
    }

    public boolean isInitialized()
    {
        return m_initialized;
    }

    /**
     * Initialize the ComponentHandler.
     */
    public void initialize()
    {
        if( m_initialized )
        {
            return;
        }

        if (m_logger.isDebugEnabled())
        {
            m_logger.debug("ComponentHandler initialized for: " + this.m_factory.getCreatedClass().getName());
        }
        m_initialized = true;
    }

    /**
     * Get a reference of the desired Component
     */
    public Component get()
        throws Exception
    {
        if( ! m_initialized )
        {
            throw new IllegalStateException( "You cannot get a component from an uninitialized holder." );
        }

        if( m_disposed )
        {
            throw new IllegalStateException( "You cannot get a component from a disposed holder" );
        }

        return (Component)m_factory.newInstance();
    }

    /**
     * Return a reference of the desired Component
     */
    public void put( final Component component )
    {
        if( !m_initialized )
        {
            throw new IllegalStateException( "You cannot put a component in an uninitialized holder." );
        }

        try
        {
            m_factory.decommission( component );
        }
        catch( final Exception e )
        {
            if (m_logger.isWarnEnabled())
            {
                m_logger.warn( "Error decommissioning component: " +
                                  m_factory.getCreatedClass().getName(), e);
            }
        }
    }

    /**
     * Dispose of the ComponentHandler and any associated Pools and Factories.
     */
    public void dispose()
    {
        try
        {
            // do nothing here

            if( m_factory instanceof Disposable )
            {
                ((Disposable)m_factory).dispose();
            }
        }
        catch( final Exception e )
        {
            if (m_logger.isWarnEnabled())
            {
                m_logger.warn( "Error decommissioning component: " +
                                  m_factory.getCreatedClass().getName(), e );
            }
        }

        m_disposed = true;
    }
}
